package com.autentia.adictos.hs.dao;

import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.Sort;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.search.FullTextQuery;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;

public class HibernateDaoImpl {

	private final SessionFactory sessionFactory;

	public HibernateDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	public void beginTransaction() {
		sessionFactory.getCurrentSession().beginTransaction();
	}

	public void commitTransaction() {
		sessionFactory.getCurrentSession().getTransaction().commit();
	}

	public void rollbackTransaction() {
		sessionFactory.getCurrentSession().getTransaction().rollback();
	}

	@SuppressWarnings("unchecked")
	public <T> List<T> loadAll(final Class<T> entityClass) {
		return (List<T>)execcute(new HibernateCallback() {

			public Object doInHibernate(Session sess) {
				final String hql = "from " + entityClass.getSimpleName();
				final Query qry = sess.createQuery(hql);
				return qry.list();
			}
		});
	}

	public void persist(final Object entity) {
		execcute(new HibernateCallback() {

			public Object doInHibernate(Session sess) {
				sess.saveOrUpdate(entity);
				return null;
			}
		});
	}

	/**
	 * Método para realizar búsquedas indexadas
	 * 
	 * @param <T>
	 * @param entityClass. Entidad base sobre la que realizar las búsquedas
	 * @param entityFields. Campos de la entidad en los que buscar
	 * @param textToFind. Texto a buscar
	 * @param analyzerName. Nombre del analizador a usar para la búsqueda. Si null entonces StandardAnalyzer
	 * @param orderBy. Campo por el que ordenar. Sólo los campos marcados como UNTOKENIZED o NO_NORM pueden ser usados para
	 *            ordenar. Si null, entonces se ordena por relevancia.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <T> List<T> findByFullText(final Class<T> entityClass, final String[] entityFields, final String textToFind,
			final String analyzerName, final String orderBy) {
		return (List<T>)execcute(new HibernateCallback() {

			public Object doInHibernate(Session sess) {
				FullTextSession fullTextSession = Search.getFullTextSession(sess);

				Analyzer analyzer = null;
				if (analyzerName != null) {
					analyzer = fullTextSession.getSearchFactory().getAnalyzer(analyzerName);
				} else {
					analyzer = new StandardAnalyzer();
				}

				final MultiFieldQueryParser parser = new MultiFieldQueryParser(entityFields, analyzer);
				final org.apache.lucene.search.Query luceneQuery;
				try {
					luceneQuery = parser.parse(textToFind);

				} catch (ParseException e) {
					System.out.println("Cannot parse [" + textToFind + "] to a full text query");
					return new ArrayList<T>(0);
				}
				final FullTextQuery query = fullTextSession.createFullTextQuery(luceneQuery, entityClass);
				if (orderBy != null)
					query.setSort(new Sort(orderBy));
				else
					query.setSort(Sort.RELEVANCE);
				return query.list();
			}
		});
	}

	private Object execcute(final HibernateCallback callback) {

		final Session sess;
		try {
			sess = sessionFactory.getCurrentSession();
		} catch (RuntimeException e) {
			System.out.println("Error recuperando la sesión");
			throw e;
		}

		boolean isInternalTransaction = false;

		Transaction tx = null;
		try {
			tx = sess.getTransaction();
		} catch (RuntimeException e) {
			System.out.println("Error recuperando la transacción");
			throw e;
		}

		try {
			if (tx == null || !tx.isActive()) {
				isInternalTransaction = true;
				tx = sess.beginTransaction();
			}
		} catch (RuntimeException e) {
			System.out.println("Error iniciando la transacción");
			throw e;
		}

		final Object callbackResult;
		try {
			callbackResult = callback.doInHibernate(sess);
		} catch (RuntimeException e) {
			tx.rollback();
			System.out.println("Error ejecutando el callBack");
			throw e;
		}

		if (isInternalTransaction) {
			tx.commit();
		}

		return callbackResult;
	}

}
